@@ -54,7 +54,7 @@
 #include <set>
 #include <vector>
 #include <memory>
-
+#include <sstream>
 
 #undef _
 #define _(a) utf8_gettext (a)
@@ -812,7 +812,8 @@ do_crypt (LPVOID arg)
       return -1;
     }
 
-  int rc = crypter->do_crypto();
+  GpgME::Error err;
+  int rc = crypter->do_crypto(err);
 
   gpgrt_lock_lock (&dtor_lock);
   if (!Mail::is_valid_ptr (mail))
@@ -823,20 +824,31 @@ do_crypt (LPVOID arg)
       return 0;
     }
 
-  if (rc == -1)
+  mail->set_window_enabled (true);
+
+  if (rc == -1 || err)
     {
       mail->reset_crypter ();
       crypter = nullptr;
-      gpgol_bug (mail->get_window (),
-                 ERR_CRYPT_RESOLVER_FAILED);
+      if (err)
+        {
+          char *buf = nullptr;
+          gpgrt_asprintf (&buf, _("Crypto operation failed:\n%s"),
+                          err.asString());
+          gpgol_message_box (mail->get_window(), buf, _("GpgOL"), MB_OK);
+          xfree (buf);
+        }
+      else
+        {
+          gpgol_bug (mail->get_window (),
+                     ERR_CRYPT_RESOLVER_FAILED);
+        }
     }
 
-  mail->set_window_enabled (true);
-
-  if (rc)
+  if (rc || err.isCanceled())
     {
-      log_debug ("%s:%s: crypto failed for: %p with: %i",
-                 SRCNAME, __func__, arg, rc);
+      log_debug ("%s:%s: crypto failed for: %p with: %i err: %i",
+                 SRCNAME, __func__, arg, rc, err.code());
       mail->set_crypt_state (Mail::NoCryptMail);
       mail->reset_crypter ();
       crypter = nullptr;
@@ -931,7 +943,7 @@ Mail::decrypt_verify()
   else if (gpgrt_asprintf (&placeholder_buf, opt.prefer_html ? decrypt_template_html :
                       decrypt_template,
                       is_smime() ? "S/MIME" : "OpenPGP",
-                      _("Encrypted message"),
+                      _("message"),
                       _("Please wait while the message is being decrypted / verified...")) == -1)
     {
       log_error ("%s:%s: Failed to format placeholder.",
@@ -2455,11 +2467,44 @@ Mail::get_sig_fpr() const
 void
 Mail::locate_keys()
 {
-  char ** recipients = get_recipients ();
-  KeyCache::instance()->startLocate (recipients);
-  KeyCache::instance()->startLocate (get_sender ().c_str ());
+  static bool locate_in_progress;
+
+  if (locate_in_progress)
+    {
+      /** XXX
+        The strangest thing seems to happen here:
+        In get_recipients the lookup for "AddressEntry" on
+        an unresolved address might cause network traffic.
+
+        So Outlook somehow "detaches" this call and keeps
+        processing window messages while the call is running.
+
+        So our do_delayed_locate might trigger a second locate.
+        If we access the OOM in this call while we access the
+        same object in the blocked "detached" call we crash.
+        (T3931)
+
+        After the window message is handled outlook retunrs
+        in the original lookup.
+
+        A better fix here might be a non recursive lock
+        of the OOM. But I expect that if we lock the handling
+        of the Windowmessage we might deadlock.
+        */
+      log_debug ("%s:%s: Locate for %p already in progress.",
+                 SRCNAME, __func__, this);
+      return;
+    }
+  locate_in_progress = true;
+
+  // First update oom data to have recipients and sender updated.
+  update_oom_data ();
+  char ** recipients = take_cached_recipients ();
   KeyCache::instance()->startLocateSecret (get_sender ().c_str ());
+  KeyCache::instance()->startLocate (get_sender ().c_str ());
+  KeyCache::instance()->startLocate (recipients);
   release_cArray (recipients);
+  locate_in_progress = false;
 }
 
 bool
@@ -2915,3 +2960,11 @@ Mail::has_crypted_or_empty_body ()
 
   return false;
 }
+
+std::string
+Mail::get_verification_result_dump()
+{
+  std::stringstream ss;
+  ss << m_verify_result;
+  return ss.str();
+}
