@@ -177,7 +177,8 @@ format_error(GpgME::DecryptionResult result, Protocol protocol)
        msg = _("Decryption canceled or timed out.");
     }
 
-  if (result.error ().code () == GPG_ERR_DECRYPT_FAILED)
+  if (result.error ().code () == GPG_ERR_DECRYPT_FAILED ||
+      result.error ().code () == GPG_ERR_NO_SECKEY)
     {
       no_sec = true;
       for (const auto &recipient: result.recipients ()) {
@@ -320,7 +321,8 @@ ParseController::parse()
         {
           verify = false;
         }
-      if (m_decrypt_result.error () || m_decrypt_result.isNull ())
+      if (m_decrypt_result.error () || m_decrypt_result.isNull () ||
+          m_decrypt_result.error ().isCanceled ())
         {
           m_error = format_error (m_decrypt_result, protocol);
         }
