@@ -25,6 +25,7 @@
 #include "oomhelp.h"
 #include "mail.h"
 #include "gpgoladdin.h"
+#include "wks-helper.h"
 
 #include <stdio.h>
 
@@ -36,7 +37,7 @@ static HWND g_responder_window = NULL;
 LONG_PTR WINAPI
 gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
-  if (message == WM_USER + 1)
+  if (message == WM_USER + 42)
     {
       wm_ctx_t *ctx = (wm_ctx_t *) lParam;
       log_debug ("%s:%s: Recieved user msg: %i",
@@ -67,6 +68,34 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
               mail->locate_keys();
               break;
             }
+          case (REVERT_MAIL):
+            {
+              auto mail = (Mail*) ctx->data;
+              if (!Mail::is_valid_ptr (mail))
+                {
+                  log_debug ("%s:%s: Revert mail for mail which is gone.",
+                             SRCNAME, __func__);
+                  break;
+                }
+
+              mail->set_needs_save (true);
+              /* Some magic here. Accessing any existing inline body cements
+                 it. Otherwise updating the body through the revert also changes
+                 the body of a inline mail. */
+              char *inlineBody = get_inline_body ();
+              xfree (inlineBody);
+
+              // Does the revert.
+              log_debug ("%s:%s: Revert mail. Invoking save.",
+                         SRCNAME, __func__);
+              invoke_oom_method (mail->item (), "Save", NULL);
+              log_debug ("%s:%s: Revert mail. Save done. Updating body..",
+                         SRCNAME, __func__);
+              mail->update_body ();
+              log_debug ("%s:%s: Revert mail done.",
+                         SRCNAME, __func__);
+              break;
+            }
           case (INVALIDATE_UI):
             {
               log_debug ("%s:%s: Invalidating UI",
@@ -76,6 +105,13 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                          SRCNAME, __func__);
               break;
             }
+          case (INVALIDATE_LAST_MAIL):
+            {
+              log_debug ("%s:%s: Invalidating last mail",
+                         SRCNAME, __func__);
+              Mail::invalidate_last_mail ();
+              break;
+            }
           case (CLOSE):
             {
               auto mail = (Mail*) ctx->data;
@@ -88,8 +124,67 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
               Mail::close (mail);
               break;
             }
+          case (CRYPTO_DONE):
+            {
+              auto mail = (Mail*) ctx->data;
+              if (!Mail::is_valid_ptr (mail))
+                {
+                  log_debug ("%s:%s: Crypto done for mail which is gone.",
+                             SRCNAME, __func__);
+                  break;
+                }
+              // modify the mail.
+              if (mail->crypt_state () == Mail::NeedsUpdateInOOM)
+                {
+                  // Save the Mail
+                  log_debug ("%s:%s: Crypto done for %p updating oom.",
+                             SRCNAME, __func__, mail);
+                  mail->update_crypt_oom();
+                }
+              if (mail->crypt_state () == Mail::NeedsSecondAfterWrite)
+                {
+                  invoke_oom_method (mail->item (), "Save", NULL);
+                  log_debug ("%s:%s: Second save done for %p Invoking second send.",
+                             SRCNAME, __func__, mail);
+                }
+              // Finaly this should pass.
+              invoke_oom_method (mail->item (), "Send", NULL);
+              log_debug ("%s:%s:  Send for %p completed.",
+                         SRCNAME, __func__, mail);
+              // Allow the WKS helper to queue a notification.
+              WKSHelper::instance()->allow_notify ();
+              log_debug ("%s:%s:  Crypto done handler completed.",
+                         SRCNAME, __func__);
+              break;
+            }
+          case (BRING_TO_FRONT):
+            {
+              // We want to avoid a race where the mail window
+              // was still the active window.
+              Sleep (250);
+              HWND wnd = get_active_hwnd ();
+              if (wnd)
+                {
+                  log_debug ("%s:%s: Bringing window %p to front.",
+                             SRCNAME, __func__, wnd);
+                  bring_to_front (wnd);
+                }
+              else
+                {
+                  log_debug ("%s:%s: No active window found for bring to front.",
+                             SRCNAME, __func__);
+                }
+              break;
+            }
+          case (WKS_NOTIFY):
+            {
+              WKSHelper::instance ()->notify ((const char *) ctx->data);
+              xfree (ctx->data);
+              break;
+            }
           default:
-            log_debug ("Unknown msg");
+            log_debug ("%s:%s: Unknown msg %x",
+                       SRCNAME, __func__, ctx->wmsg_type);
         }
         return DefWindowProc(hWnd, message, wParam, lParam);
     }
@@ -140,7 +235,7 @@ send_msg_to_ui_thread (wm_ctx_t *ctx)
                SRCNAME, __func__);
     return -1;
   }
-  SendMessage (responder, WM_USER + 1, 0, (LPARAM) ctx);
+  SendMessage (responder, WM_USER + 42, 0, (LPARAM) ctx);
   return 0;
 }
 
@@ -157,6 +252,27 @@ do_in_ui_thread (gpgol_wmsg_type type, void *data)
   return ctx.err;
 }
 
+static DWORD WINAPI
+do_async (LPVOID arg)
+{
+  wm_ctx_t *ctx = (wm_ctx_t*) arg;
+  log_debug ("%s:%s: Do async with type %i",
+             SRCNAME, __func__, ctx ? ctx->wmsg_type : -1);
+  send_msg_to_ui_thread (ctx);
+  xfree (ctx);
+  return 0;
+}
+
+void
+do_in_ui_thread_async (gpgol_wmsg_type type, void *data)
+{
+  wm_ctx_t *ctx = (wm_ctx_t *) calloc (1, sizeof (wm_ctx_t));
+  ctx->wmsg_type = type;
+  ctx->data = data;
+
+  CloseHandle (CreateThread (NULL, 0, do_async, (LPVOID) ctx, 0, NULL));
+}
+
 static std::vector <LPDISPATCH> explorers;
 
 void
@@ -244,7 +360,8 @@ create_message_hook()
                            GetCurrentThreadId());
 }
 
-GPGRT_LOCK_DEFINE(invalidate_lock);
+gpgrt_lock_t invalidate_lock = GPGRT_LOCK_INITIALIZER;
+
 static bool invalidation_in_progress;
 
 DWORD WINAPI
@@ -261,7 +378,7 @@ delayed_invalidate_ui (LPVOID)
   /* We sleep here a bit to prevent invalidation immediately
      after the selection change before we have started processing
      the mail. */
-  Sleep (500);
+  Sleep (250);
   do_in_ui_thread (INVALIDATE_UI, nullptr);
   invalidation_in_progress = false;
   gpgrt_lock_unlock(&invalidate_lock);
